function s = slet2(x)
%
% two-dimensional slantlet transform
%
% % example
% x = randn(2^5);
% s = slet2(x);
% y = islet2(s);
% max(max(abs(x-y)))


sz = size(x);
if sz(1) ~= sz(2)
    disp('sorry, need x square.')
    s = [];
    return
end
N = length(x);
l = log2(N);
if l ~= round(l)
    disp('sorry, need length(x) a power of 2.')
    s = [];
    return
end

u00 = x(1:2:N,1:2:N) + x(1:2:N,2:2:N) + x(2:2:N,1:2:N) + x(2:2:N,2:2:N);
u01 = x(1:2:N,2:2:N) + x(2:2:N,2:2:N);
u10 = x(2:2:N,1:2:N) + x(2:2:N,2:2:N);
u11 = x(2:2:N,2:2:N);
L = N/2;		% length of "moment" vectors

s = zeros(size(x));
for i = 1:l-1
    [a0,a1,b0,b1,a0r,a1r,b0r,b1r] = getg(i);
    [c0,c1,c0r,c1r] = gets(i);

    k0 = 1:2:L;
    k1 = 2:2:L;

    % u00A0: u00(n1,2*n2;i+1,i), similarly for u01A0, u10A0, u11A0
    % u00A1: u00(n1,2*n2+1;i+1,i), similarly for u01A1, u10A1, u11A1
    u00A0 = u00(k0,k0) + u00(k1,k0);
    u00A1 = u00(k0,k1) + u00(k1,k1);
    u01A0 = u01(k0,k0) + u01(k1,k0);
    u01A1 = u01(k0,k1) + u01(k1,k1);
    u10A0 = u10(k0,k0) + 2^i*u00(k1,k0) + u10(k1,k0);
    u10A1 = u10(k0,k1) + 2^i*u00(k1,k1) + u10(k1,k1);
    u11A0 = u11(k0,k0) + 2^i*u01(k1,k0) + u11(k1,k0);
    u11A1 = u11(k0,k1) + 2^i*u01(k1,k1) + u11(k1,k1);

    % u00B0: u00(2*n1,n2;i,i+1), similarly for u01B0, u10B0, u11B0
    % u00B1: u00(2*n1+1,n2;i,i+1), similarly for u01B1, u10B1, u11B1
    u00B0 = u00(k0,k0) + u00(k0,k1);
    u00B1 = u00(k1,k0) + u00(k1,k1);
    u01B0 = u01(k0,k0) + 2^i*u00(k0,k1) + u01(k0,k1);
    u01B1 = u01(k1,k0) + 2^i*u00(k1,k1) + u01(k1,k1);
    u10B0 = u10(k0,k0) + u10(k0,k1);
    u10B1 = u10(k1,k0) + u10(k1,k1);
    u11B0 = u11(k0,k0) + 2^i*u10(k0,k1) + u11(k0,k1);
    u11B1 = u11(k1,k0) + 2^i*u10(k1,k1) + u11(k1,k1);

    % low-high

    tmp1 = a0*u00A0 + a1*u01A0 + b0*u00A1 + b1*u01A1;
    tmp2 = a0*u10A0 + a1*u11A0 + b0*u10A1 + b1*u11A1;
    s(L+k0,k0) = c0*tmp1 + c1*tmp2;		% s,g
    s(L+k1,k0) = c0r*tmp1 + c1r*tmp2;	% sr,g

    tmp1 = a0r*u00A0 + a1r*u01A0 + b0r*u00A1 + b1r*u01A1;
    tmp2 = a0r*u10A0 + a1r*u11A0 + b0r*u10A1 + b1r*u11A1;
    s(L+k0,k1) = c0*tmp1 + c1*tmp2;		% s,gr
    s(L+k1,k1) = c0r*tmp1 + c1r*tmp2;	% sr,gr

    % high-low

    tmp1 = a0*u00B0 + a1*u10B0 + b0*u00B1 + b1*u10B1;
    tmp2 = a0*u01B0 + a1*u11B0 + b0*u01B1 + b1*u11B1;
    s(k0,L+k0) = c0*tmp1 + c1*tmp2;		% g,s
    s(k0,L+k1) = c0r*tmp1 + c1r*tmp2;	% g,sr

    tmp1 = a0r*u00B0 + a1r*u10B0 + b0r*u00B1 + b1r*u10B1;
    tmp2 = a0r*u01B0 + a1r*u11B0 + b0r*u01B1 + b1r*u11B1;
    s(k1,L+k0) = c0*tmp1 + c1*tmp2;		% gr,s
    s(k1,L+k1) = c0r*tmp1 + c1r*tmp2;	% gr,sr

    % high-high

    tmp1 = a0*u00(k0,k0)+a1*u01(k0,k0)+b0*u00(k0,k1)+b1*u01(k0,k1);
    tmp2 = a0*u10(k0,k0)+a1*u11(k0,k0)+b0*u10(k0,k1)+b1*u11(k0,k1);
    tmp3 = a0*u00(k1,k0)+a1*u01(k1,k0)+b0*u00(k1,k1)+b1*u01(k1,k1);
    tmp4 = a0*u10(k1,k0)+a1*u11(k1,k0)+b0*u10(k1,k1)+b1*u11(k1,k1);
    s(L+k0,L+k0) = a0*tmp1 + a1*tmp2 + b0*tmp3 + b1*tmp4;		% g,g
    s(L+k1,L+k0) = a0r*tmp1 + a1r*tmp2 + b0r*tmp3 + b1r*tmp4;	% gr,g

    tmp1 = a0r*u00(k0,k0)+a1r*u01(k0,k0)+b0r*u00(k0,k1)+b1r*u01(k0,k1);
    tmp2 = a0r*u10(k0,k0)+a1r*u11(k0,k0)+b0r*u10(k0,k1)+b1r*u11(k0,k1);
    tmp3 = a0r*u00(k1,k0)+a1r*u01(k1,k0)+b0r*u00(k1,k1)+b1r*u01(k1,k1);
    tmp4 = a0r*u10(k1,k0)+a1r*u11(k1,k0)+b0r*u10(k1,k1)+b1r*u11(k1,k1);
    s(L+k0,L+k1) = a0*tmp1 + a1*tmp2 + b0*tmp3 + b1*tmp4;		% g,gr
    s(L+k1,L+k1) = a0r*tmp1 + a1r*tmp2 + b0r*tmp3 + b1r*tmp4;	% gr,gr

    % update moments

    u00 = u00A0 + u00A1;
    u01 = u01A0 + 2^i*u00A1 + u01A1;
    u10 = u10A0 + u10A1;
    u11 = u11A0 + 2^i*u10A1 + u11A1;

    L = L/2;
end


% DC and Linear components

[a0,a1,b0,b1,c0,c1,d0,d1] = gethf(l);

% low-low
hh = u00*(a0*a0+a0*b0+b0*a0+b0*b0);
% the other times are not needed becuase
% (a0*a1+a0*b1+b0*a1+b0*b1) = 0
% (a1*a0+a1*b0+b1*a0+b1*b0) = 0
% (a1*a1+a1*b1+b1*a1+b1*b1) = 0

% low-hi
hg = u00*(a0*c0+a0*d0+b0*c0+b0*d0) + u01*(a0*c1+a0*d1+b0*c1+b0*d1);
% hg = ...
% u00*(a0*c0+a0*d0+b0*c0+b0*d0) + ...
% u01*(a0*c1+a0*d1+b0*c1+b0*d1) + ...
% u10*(a1*c0+a1*d0+b1*c0+b1*d0) + ...
% u11*(a1*c1+a1*d1+b1*c1+b1*d1);
% but (a1*c0+a1*d0+b1*c0+b1*d0)=0 and (a1*c1+a1*d1+b1*c1+b1*d1)=0

% hi-low
gh = u00*(c0*a0+c0*b0+d0*a0+d0*b0) + u10*(c1*a0+c1*b0+d1*a0+d1*b0);
% gh = ...
% u00*(c0*a0+c0*b0+d0*a0+d0*b0) + ...
% u01*(c0*a1+c0*b1+d0*a1+d0*b1) + ...
% u10*(c1*a0+c1*b0+d1*a0+d1*b0) + ...
% u11*(c1*a1+c1*b1+d1*a1+d1*b1);
% but (c0*a1+c0*b1+d0*a1+d0*b1)=0 and (c1*a1+c1*b1+d1*a1+d1*b1)=0

% hi-hi
gg = ...
    u00*(c0*c0+c0*d0+d0*c0+d0*d0) + ...
    u01*(c0*c1+c0*d1+d0*c1+d0*d1) + ...
    u10*(c1*c0+c1*d0+d1*c0+d1*d0) + ...
    u11*(c1*c1+c1*d1+d1*c1+d1*d1);


s(1,1) = hh;
s(1,2) = hg;
s(2,1) = gh;
s(2,2) = gg;
